local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local SharedMedia = LibStub("LibSharedMedia-3.0")
local Assert = BaikCore.Assert
local Database = BaikCore.Database
local Option = BaikCore.Option

-- Add Module
local StatusBar = BaikCore:NewModule("StatusBar")
BaikCore.StatusBar = StatusBar

-- Constants
local MEDIA_TYPE = SharedMedia.MediaType.STATUSBAR
local GLOBAL_DB_ENABLED = "global_texture_on"
local GLOBAL_DB_BAR = "global_texture"

StatusBar.ENABLED_DEFAULT = false
StatusBar.DEFAULT = SharedMedia:GetDefault(MEDIA_TYPE)

-- Private Variables
local frames = {}

-- Private Methods
local function StatusBarName(dbValue)
    Assert:String(dbValue)

    if Database:Get(GLOBAL_DB_ENABLED) then
        return Database:Get(GLOBAL_DB_BAR)
    end

    return Database:Get(dbValue)
end

local function StatusBarTexture(dbValue)
    local name = StatusBarName(dbValue)

    return SharedMedia:Fetch(MEDIA_TYPE, name)
end

local function SyncGlobal()
    local texture = StatusBarTexture(GLOBAL_DB_BAR)

    for _, frameTable in pairs(frames) do
        for _, frame in pairs(frameTable) do
            if frame then
                frame:SetStatusBarTexture(texture)
            end
        end
    end
end

local function SyncAll()
    for dbValue, frameTable in pairs(frames) do
        StatusBar:Sync(dbValue)
    end
end

local function GetOpts()
    return {
        name = "StatusBar",
        type = "group",
        args = {
            enabled = {
                name = "Enable",
                type = "toggle",
                order = 0,
                get = function(...)
                    return Database:Get(GLOBAL_DB_ENABLED)
                end,
                set = function(self, value, ...)
                    Database:Set(GLOBAL_DB_ENABLED, value)
                    if value then
                        SyncGlobal()
                    else
                        SyncAll()
                    end
                end
            },
            texture = {
                name = "Texture",
                type = "select",
                disabled = function(...)
                    return not Database:Get(GLOBAL_DB_ENABLED)
                end,
                dialogControl = "LSM30_Statusbar",
                values = AceGUIWidgetLSMlists.statusbar,
                get = function(...)
                    return Database:Get(GLOBAL_DB_BAR)
                end,
                set = function(self, value, ...)
                    Assert:String(value)
                    Database:Set(GLOBAL_DB_BAR, value)
                    SyncGlobal()
                end
            }
        }
    }
end

-- StatusBar API
function StatusBar:AddBar(bar, dbValue)
    Assert:String(dbValue)

    if not frames[dbValue] then
        frames[dbValue] = {}
    end

    table.insert(frames[dbValue], bar)
end

function StatusBar:AddBars(barTable, dbValue)
    Assert:Table(barTable)
    Assert:String(dbValue)

    if not frames[dbValue] then
        frames[dbValue] = {}
    end

    for _, bar in pairs(barTable) do
        table.insert(frames[dbValue], bar)
    end
end

function StatusBar:ClearBars(dbValue)
    frames[dbValue] = {}
end

function StatusBar:Sync(dbValue)
    Assert:String(dbValue)
    Assert:NotNull(frames[dbValue])

    local barTexture = StatusBarTexture(dbValue)
    for _, frame in pairs(frames[dbValue]) do
        if frame then
            frame:SetStatusBarTexture(barTexture)
        end
    end
end

function StatusBar:SyncBar(bar, dbValue)
    Assert:String(dbValue)
    Assert:NotNull(bar)
    Assert:NotNull(frames[dbValue])

    local barTexture = StatusBarTexture(dbValue)
    bar:SetStatusBarTexture(barTexture)
end

function StatusBar:GetOpts(dbValue)
    Assert:String(dbValue)
    return {
        name = "Texture",
        type = "select",
        disabled = function(...)
            return Database:Get(GLOBAL_DB_ENABLED)
        end,
        dialogControl = "LSM30_Statusbar",
        values = AceGUIWidgetLSMlists.statusbar,
        get = function(...)
            return Database:Get(dbValue)
        end,
        set = function(self, value, ...)
            Database:Set(dbValue, value)
            StatusBar:Sync(dbValue)
        end
    }
end

-- Ace3 Callbacks
function StatusBar:OnInitialize()
    Database:InitIfEmpty(GLOBAL_DB_ENABLED, StatusBar.ENABLED_DEFAULT)
    Database:InitIfEmpty(GLOBAL_DB_BAR, StatusBar.DEFAULT)
    Option:AddOption(Option.NAME, "status_bar", GetOpts())
end

function StatusBar:OnEnable()
end

function StatusBar:OnDisable()
end
