local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local Assert = BaikCore.Assert
local Database = BaikCore.Database
local Option = BaikCore.Option
local OrderNumber = BaikCore.OrderNumber
local Ui = BaikCore.Ui

-- Add Module
local VertexColor = BaikCore:NewModule("VertexColor")
BaikCore.VertexColor = VertexColor

-- Constants
local OPTION_NAME = "vertex_color"
local SUFFIX_DB_ENABLED = "%s_color_on"
local SUFFIX_DB_COLOR = "%s_color"
local SUFFIX_DB_CLASS = "%s_color_class"
local SUFFIX_DB_CLASS_ALPHA = "%s_color_class_alpha"

local PREFIX_DB = "global"
local GLOBAL_DB_ENABLED = string.format(SUFFIX_DB_ENABLED, PREFIX_DB)
local GLOBAL_DB_COLOR = string.format(SUFFIX_DB_COLOR, PREFIX_DB)
local GLOBAL_DB_CLASS = string.format(SUFFIX_DB_CLASS, PREFIX_DB)
local GLOBAL_DB_CLASS_ALPHA = string.format(SUFFIX_DB_CLASS_ALPHA, PREFIX_DB)

VertexColor.DEFAULT_ENABLE = false
VertexColor.DEFAULT_COLOR = Ui:CreateColor(1, 1, 1, 1)
VertexColor.DEFAULT_CLASS = false
VertexColor.DEFAULT_CLASS_ALPHA = 1

local CLASS_COLOR = nil

-- Private Variables
local colorFunc = {}

-- Private Methods
local function GetDbEnable(dbPrefix)
    return string.format(SUFFIX_DB_ENABLED, dbPrefix)
end

local function GetDbColor(dbPrefix)
    return string.format(SUFFIX_DB_COLOR, dbPrefix)
end

local function GetDbClass(dbPrefix)
    return string.format(SUFFIX_DB_CLASS, dbPrefix)
end

local function GetDbAlpha(dbPrefix)
    return string.format(SUFFIX_DB_CLASS_ALPHA, dbPrefix)
end

local function GetGlobalEnabled()
    return Database:Get(GLOBAL_DB_ENABLED)
end

local function GetPrefix(prefix)
    local globalEnabled = GetGlobalEnabled()

    return globalEnabled and PREFIX_DB or prefix
end

local function GetEnabled(dbPrefix)
    local prefix = GetPrefix(dbPrefix)
    local enable = GetDbEnable(prefix)

    return Database:Get(enable)
end

local function GetClass(dbPrefix)
    local prefix = GetPrefix(dbPrefix)
    local class = GetDbClass(dbPrefix)

    return Database:Get(class)
end

local function GetColor(dbPrefix)
    local prefix = GetPrefix(dbPrefix)
    local class = GetDbClass(prefix)

    if Database:Get(class) then
        -- TODO - Add target parameter
        local alpha = GetDbAlpha(prefix)
        local classColor = Ui:GetClassColor("player")
        classColor.a = Database:Get(alpha)

        return classColor
    end

    local color = GetDbColor(prefix)
    return Database:Get(color)
end

local function ColorFrameInner(frame, dbPrefix)
    local color = GetColor(dbPrefix)
    frame:SetVertexColor(color.r, color.g, color.b, color.a)
end

local function Setup()
    for _, func in pairs(colorFunc) do
        if func ~= Setup then
            func()
        end
    end
end

local function IsGlobal(dbPrefix)
    return dbPrefix == PREFIX_DB
end

local function GetEnableOpts(dbPrefix, orderNumber)
    local dbEnable = GetDbEnable(dbPrefix)

    local function GlobalDisable()
        return false
    end

    local function Disable()
        return GetGlobalEnabled()
    end

    local disableFunc = IsGlobal(dbPrefix) and GlobalDisable or Disable

    return {
        name = "Enable",
        type = "toggle",
        order = orderNumber:GetNext(),
        disabled = disableFunc,
        get = function(...)
            return Database:Get(dbEnable)
        end,
        set = function(self, value, ...)
            Database:Set(dbEnable, value)

            colorFunc[dbPrefix]()
        end
    }
end

local function GetColorOpts(dbPrefix, orderNumber)
    local dbColor = GetDbColor(dbPrefix)
    local dbClass = GetDbClass(dbPrefix)

    local function GlobalDisable()
        return not GetGlobalEnabled() or Database:Get(dbClass)
    end

    local function Disable()
        return GetGlobalEnabled() or not GetEnabled(dbPrefix) or Database:Get(dbClass)
    end

    local disableFunc = IsGlobal(dbPrefix) and GlobalDisable or Disable

    return {
        name = "Color",
        type = "color",
        order = orderNumber:GetNext(),
        hasAlpha = true,
        disabled = disableFunc,
        get = function(...)
            local color = Database:Get(dbColor)
            return color.r, color.g, color.b, color.a
        end,
        set = function(self, r, g, b, a, ...)
            Assert:Number(r)
            Assert:Number(g)
            Assert:Number(b)
            Assert:Number(a)
            Database:Set(dbColor, Ui:CreateColor(r, g, b, a))

            colorFunc[dbPrefix]()
        end
    }
end

local function GetClassOpts(dbPrefix, orderNumber)
    local dbClass = GetDbClass(dbPrefix)

    local function GlobalDisable()
        return not GetGlobalEnabled()
    end

    local function Disable()
        return GetGlobalEnabled() or not GetEnabled(dbPrefix)
    end

    local disableFunc = IsGlobal(dbPrefix) and GlobalDisable or Disable

    return {
        name = "Class Color",
        type = "toggle",
        order = orderNumber:GetNext(),
        disabled = disableFunc,
        get = function(...)
            return Database:Get(dbClass)
        end,
        set = function(self, value, ...)
            Assert:Boolean(value)
            Database:Set(dbClass, value)

            colorFunc[dbPrefix]()
        end
    }
end

local function GetAlphaOpts(dbPrefix, orderNumber)
    local dbAlpha = GetDbAlpha(dbPrefix)
    local dbClass = GetDbClass(dbPrefix)

    local function GlobalDisable()
        return not (GetGlobalEnabled() and Database:Get(dbClass))
    end

    local function Disable()
        return GetGlobalEnabled() or not (GetEnabled(dbPrefix) and Database:Get(dbClass))
    end

    local disableFunc = IsGlobal(dbPrefix) and GlobalDisable or Disable

    return {
        name = "Class Alpha",
        type = "range",
        order = orderNumber:GetNext(),
        min = 0,
        max = 1,
        isPercent = true,
        disabled = disableFunc,
        get = function(...)
            return Database:Get(dbAlpha)
        end,
        set = function(self, value, ...)
            Assert:Number(value)
            Database:Set(dbAlpha, value)
            colorFunc[dbPrefix]()
        end
    }
end

local function GetOpts()
    local opts = {
        name = "Art",
        type = "group",
        args = {
            enable = GetEnableOpts()
        }
    }
end

-- Public API
function VertexColor:ColorFrame(frame, dbPrefix)
    Assert:Table(frames)
    Assert:String(dbPrefix)

    if not GetEnabled(dbPrefix) then
        return
    end

    ColorFrameInner(frame, dbPrefix)
end

function VertexColor:ColorFrameAll(frames, dbPrefix)
    Assert:Table(frames)
    Assert:String(dbPrefix)

    if not GetEnabled(dbPrefix) then
        return
    end

    for _, frame in pairs(frames) do
        ColorFrameInner(frame, dbPrefix)
    end
end

function VertexColor:Sync(dbPrefix)
    Assert:String(dbPrefix)
    -- Ensure we have frames to sync
    if not colorFunc[dbPrefix] then
        Log:w(string.format("%s does not have a color sync function", dbValue))
        return
    end

    colorFunc[dbValue]()
end

function VertexColor:InsertOpts(argTbl, dbPrefix, func, orderNumber)
    Assert:Table(argTbl)
    Assert:String(dbPrefix)
    Assert:Function(func)

    if not orderNumber then
        orderNumber = OrderNumber:Create()
    end

    colorFunc[dbPrefix] = func
    argTbl["enabled"] = GetEnableOpts(dbPrefix, orderNumber)
    argTbl["color"] = GetColorOpts(dbPrefix, orderNumber)
    argTbl["class"] = GetClassOpts(dbPrefix, orderNumber)
    argTbl["class_alpha"] = GetAlphaOpts(dbPrefix, orderNumber)
end

function VertexColor:GetOpts(dbPrefix, func)
    Assert:String(dbPrefix)
    Assert:Function(func)

    local opts = {
        name = "Art",
        type = "group",
        args = {
        }
    }
    self:InsertOpts(opts.args, dbPrefix, func)

    return opts
end


function VertexColor:InitDb(dbPrefix)
    Assert:String(dbPrefix)

    Database:InitIfEmpty(GetDbEnable(dbPrefix), self.DEFAULT_ENABLE)
    Database:InitIfEmpty(GetDbColor(dbPrefix), self.DEFAULT_COLOR)
    Database:InitIfEmpty(GetDbClass(dbPrefix), self.DEFAULT_CLASS)
    Database:InitIfEmpty(GetDbAlpha(dbPrefix), self.DEFAULT_CLASS_ALPHA)
end

-- Ace3 Callbacks
function VertexColor:OnInitialize()
    self:InitDb(PREFIX_DB)
    Option:AddOption(Option.NAME, OPTION_NAME, self:GetOpts(PREFIX_DB, Setup))
end

function VertexColor:OnEnable()
end

function VertexColor:OnDisable()
end
