local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local Assert = BaikCore.Assert
local CombatSafe = BaikCore.CombatSafe
local Database = BaikCore.Database
local Hook = BaikCore.Hook
local Ui = BaikCore.Ui

-- Add Module
local Visibility = BaikCore:NewModule("Visibility", "AceHook-3.0")
BaikCore.Visibility = Visibility
Visibility.DEFAULT = 3

-- Constants
local STATE_OPTS = {
    "Hide",
    "Hover",
    "Show"
}

local HIDE_IDX = 1
local HOVER_IDX = 2
local SHOW_IDX = 3

local SUFFIX_DB_VIS = "%s_visibility"

-- Private Variables
local visFunc = {}

-- Private Methods
local function GetDbVis(dbPrefix)
    return string.format(SUFFIX_DB_VIS, dbPrefix)
end

local function GetVis(dbPrefix)
    local dbVis = GetDbVis(dbPrefix)

    return Database:Get(dbVis)
end

local function HookHover(frame, parent)
    Hook:EnsureHookScript(Visibility, frame, "OnEnter", function(...) parent:SetAlpha(1) end)
    Hook:EnsureHookScript(Visibility, frame, "OnLeave", function(...) parent:SetAlpha(0) end)
end

local function UnhookHover(frame)
    Hook:EnsureUnhook(Visibility, frame, "OnEnter")
    Hook:EnsureUnhook(Visibility, frame, "OnLeave")
end

local function SetVisibilityLocked(dbPrefix, frame, parent, useAlpha)
    local visibility = GetVis(dbPrefix)

    if not parent then
        parent = frame
    end

    if visibility == HIDE_IDX then
        if useAlpha then
            parent:SetAlpha(0)
        else
            Ui:HideFinal(parent)
        end
        UnhookHover(frame)
    elseif visibility == HOVER_IDX then
        Ui:UndoHideFinal(parent)
        parent:SetAlpha(0)
        HookHover(frame, parent)
    else
        Ui:UndoHideFinal(parent)
        parent:SetAlpha(1)
        UnhookHover(frame)
    end
end

local function SetupLocked()
    for _, func in pairs(visFunc) do
        func()
    end
end

local function Setup()
    if CombatSafe:InLockdown() then
        return
    end

    SetupLocked()
end

-- Visibility API
function Visibility:SetupFrame(dbPrefix, frame, parent)
    Assert:String(dbPrefix)
    Assert:Table(frame)

    if CombatSafe:InLockdown() then
        return
    end

    SetVisibilityLocked(dbPrefix, frame, parent)
end

function Visibility:SetupFrameAll(dbPrefix, frameTbl, parent)
    Assert:String(dbPrefix)
    Assert:Table(frameTbl)

    if CombatSafe:InLockdown() then
        return
    end

    for _, frame in pairs(frameTbl) do
        SetVisibilityLocked(dbPrefix, frame, parent)
    end
end

function Visibility:SetupFrameAlpha(dbPrefix, frame, parent)
    Assert:String(dbPrefix)
    Assert:Table(frame)

    if CombatSafe:InLockdown() then
        return
    end

    SetVisibilityLocked(dbPrefix, frame, parent, true)
end

function Visibility:SetupFrameAllAlpha(dbPrefix, frameTbl, parent)
    Assert:String(dbPrefix)
    Assert:Table(frameTbl)

    if CombatSafe:InLockdown() then
        return
    end

    for _, frame in pairs(frameTbl) do
        SetVisibilityLocked(dbPrefix, frame, parent, true)
    end
end

function Visibility:GetOpts(dbPrefix, func)
    visFunc[dbPrefix] = func
    local dbVis = GetDbVis(dbPrefix)

    return {
        name = "Visibility",
        type = "select",
        desc = "Adjusts frame visibility",
        values = STATE_OPTS,
        get = function(...)
            return Database:Get(dbVis)
        end,
        set = function(self, key, value)
            local cur = Database:Get(dbVis)

            -- Update if state changed
            if cur ~= key then
                Database:Set(dbVis, key)

                if not CombatSafe:InLockdown() then
                    func()
                end
            end
        end,
    }
end

function Visibility:InitDb(dbPrefix)
    Assert:String(dbPrefix)
    local dbVis = GetDbVis(dbPrefix)

    Database:InitIfEmpty(dbVis, self.DEFAULT)
end

-- Ace3 Callbacks
function Visibility:OnInitialize()
    CombatSafe:Setup(SetupLocked)
end

function Visibility:OnEnable()
end

function Visibility:OnDisable()
end
